package obs.samples;

import android.app.Activity;
import android.os.AsyncTask;
import android.os.Bundle;
import android.text.method.ScrollingMovementMethod;
import android.view.View;
import android.widget.TextView;

import com.obs.services.ObsClient;
import com.obs.services.ObsConfiguration;
import com.obs.services.exception.ObsException;
import com.obs.services.internal.ServiceException;
import com.obs.services.internal.utils.ConvertUtil;
import com.obs.services.internal.utils.ServiceUtils;
import com.obs.services.model.BucketCorsRule;
import com.obs.services.model.HttpMethodEnum;
import com.obs.services.model.S3BucketCors;
import com.obs.services.model.SpecialParamEnum;
import com.obs.services.model.V4TemporarySignatureRequest;
import com.obs.services.model.V4TemporarySignatureResponse;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import okhttp3.Call;
import okhttp3.MediaType;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.RequestBody;
import okhttp3.Response;

/**
 * This sample demonstrates how to do common operations in v4 temporary signature way
 * on OBS using the OBS SDK for Android.
 */
public class V4TemporarySignatureSample extends Activity
{
    
    private static final String endPoint = "yourdomainname";
    
    private static final String ak = "*** Provide your Access Key ***";
    
    private static final String sk = "*** Provide your Secret Key ***";
    
    private static String bucketName = "my-obs-bucket-demo";
    
    private static String objectKey = "my-obs-object-key-demo";
    
    private static ObsClient obsClient;
    
    private static StringBuffer sb;
    
    private static OkHttpClient httpClient;
    
    @Override
    protected void onCreate(Bundle savedInstanceState)
    {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);
        
        sb = new StringBuffer();
        
        httpClient = new OkHttpClient.Builder().followRedirects(false)
            .retryOnConnectionFailure(false)
            .cache(null)
            .build();
        
        ObsConfiguration config = new ObsConfiguration();
        config.setEndPoint(endPoint);
        config.setHttpsOnly(false);
        
        /*
        * Constructs a obs client instance with your account for accessing OBS
        */
        obsClient = new ObsClient(ak, sk, config);
        final TextView tv = (TextView)findViewById(R.id.tv);
        tv.setText("Click to start test");
        
        tv.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View v)
            {
                tv.setClickable(false);
                AsyncTask<Void, Void, String> task = new V4TemporarySignatureTask();
                task.execute();
            }
        });
    }
    
    class V4TemporarySignatureTask extends AsyncTask<Void, Void, String>
    {
        
        @Override
        protected String doInBackground(Void... params)
        {
            try
            {
                
                /*
                * Create bucket
                */
                doCreateBucket();
                
                /*
                 * Set/Get/Delete bucket cors
                 */
                doBucketCorsOperations();
                
                /*
                 * Create object
                 */
                doCreateObject();
                
                /*
                 * Get object
                 */
                doGetObject();
                
                /*
                 * Set/Get object acl
                 */
                doObjectAclOperations();
                
                /*
                 * Delete object
                 */
                doDeleteObject();
                
                /*
                 * Delete bucket
                 */
                doDeleteBucket();
                
                return sb.toString();
            }
            catch (ObsException e)
            {
                sb.append("\n\n");
                sb.append("Response Code:" + e.getResponseCode())
                    .append("\n\n")
                    .append("Error Message:" + e.getErrorMessage())
                    .append("\n\n")
                    .append("Error Code:" + e.getErrorCode())
                    .append("\n\n")
                    .append("Request ID:" + e.getErrorRequestId())
                    .append("\n\n")
                    .append("Host ID:" + e.getErrorHostId());
                return sb.toString();
            }
            catch (Exception e)
            {
                sb.append("\n\n");
                sb.append(e.getMessage());
                return sb.toString();
            }
            finally
            {
                if (obsClient != null)
                {
                    try
                    {
                        /*
                         * Close obs client
                         */
                        obsClient.close();
                    }
                    catch (IOException e)
                    {
                    }
                }
            }
            
        }
        
        @Override
        protected void onPostExecute(String result)
        {
            TextView tv = (TextView)findViewById(R.id.tv);
            tv.setText(result);
            tv.setOnClickListener(null);
            tv.setMovementMethod(ScrollingMovementMethod.getInstance());
        }
        
        private void doDeleteBucket()
            throws ObsException, IOException
        {
            V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.DELETE, 300);
            v4Request.setBucketName(bucketName);
            V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
            sb.append("Deleting bucket using v4 temporary signature url:");
            sb.append("\t" + v4Response.getSignedUrl() + "\n");
            Request.Builder builder = new Request.Builder();
            builder.url(v4Response.getSignedUrl());
            for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
            {
                builder.header(entry.getKey(), entry.getValue().toString());
            }
            getResponse(builder.delete().build());
            sb.append("\n\n");
        }
        
        private void doDeleteObject()
            throws ObsException, IOException
        {
            V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.DELETE, 300);
            v4Request.setBucketName(bucketName);
            v4Request.setObjectKey(objectKey);
            V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
            sb.append("Deleting object using v4 temporary signature url:");
            sb.append("\t" + v4Response.getSignedUrl() + "\n");
            Request.Builder builder = new Request.Builder();
            builder.url(v4Response.getSignedUrl());
            for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
            {
                builder.header(entry.getKey(), entry.getValue().toString());
            }
            getResponse(builder.delete().build());
            sb.append("\n\n");
        }
        
        private void doObjectAclOperations()
            throws IOException, ObsException
        {
            Map<String, Object> headers = new HashMap<String, Object>();
            headers.put("x-amz-acl", "public-read");
            
            V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.PUT, 300);
            v4Request.setBucketName(bucketName);
            v4Request.setObjectKey(objectKey);
            v4Request.setHeaders(headers);
            v4Request.setSpecialParam(SpecialParamEnum.ACL);
            V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
            
            sb.append("Setting object ACL to public-read using v4 temporary signature url:");
            sb.append("\t" + v4Response.getSignedUrl() + "\n");
            Request.Builder builder = new Request.Builder();
            builder.url(v4Response.getSignedUrl());
            for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
            {
                builder.header(entry.getKey(), entry.getValue().toString());
            }
            builder.put(RequestBody.create(null, "".getBytes("UTF-8")));
            getResponse(builder.build());
            sb.append("\n\n");
            
            v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.GET, 300);
            v4Request.setBucketName(bucketName);
            v4Request.setObjectKey(objectKey);
            v4Request.setSpecialParam(SpecialParamEnum.ACL);
            v4Response = obsClient.createV4TemporarySignature(v4Request);
            
            sb.append("Getting object ACL using v4 temporary signature url:");
            sb.append("\t" + v4Response.getSignedUrl() + "\n");
            
            builder = new Request.Builder();
            builder.url(v4Response.getSignedUrl());
            for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
            {
                builder.header(entry.getKey(), entry.getValue().toString());
            }
            getResponse(builder.get().build());
            sb.append("\n\n");
        }
        
        private void doGetObject()
            throws IOException, ObsException
        {
            V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.GET, 300);
            v4Request.setBucketName(bucketName);
            v4Request.setObjectKey(objectKey);
            V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
            sb.append("Getting object using v4 temporary signature url:");
            sb.append("\t" + v4Response.getSignedUrl() + "\n");
            Request.Builder builder = new Request.Builder();
            builder.url(v4Response.getSignedUrl());
            for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
            {
                builder.header(entry.getKey(), entry.getValue().toString());
            }
            getResponse(builder.get().build());
            sb.append("\n\n");
        }
        
        private void doCreateObject()
            throws IOException, ObsException
        {
            Map<String, Object> headers = new HashMap<String, Object>();
            headers.put("Content-Type", "text/plain");
            
            V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.PUT, 300);
            v4Request.setBucketName(bucketName);
            v4Request.setObjectKey(objectKey);
            v4Request.setHeaders(headers);
            V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
            
            sb.append("Createing object using v4 temporary signature url:");
            sb.append("\t" + v4Response.getSignedUrl() + "\n");
            Request.Builder builder = new Request.Builder();
            builder.url(v4Response.getSignedUrl());
            for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
            {
                builder.header(entry.getKey(), entry.getValue().toString());
            }
            builder.put(RequestBody.create(MediaType.parse("text/plain"), "Hello OBS".getBytes("UTF-8")));
            getResponse(builder.build());
            sb.append("\n\n");
            
        }
        
        private void doBucketCorsOperations()
            throws IOException, ServiceException, ObsException
        {
            S3BucketCors s3BucketCors = new S3BucketCors();
            BucketCorsRule rule = new BucketCorsRule();
            rule.getAllowedHeader().add("Authorization");
            rule.getAllowedOrigin().add("http://www.a.com");
            rule.getAllowedOrigin().add("http://www.b.com");
            rule.getExposeHeader().add("x-obs-test1");
            rule.getExposeHeader().add("x-obs-test2");
            rule.setMaxAgeSecond(100);
            rule.getAllowedMethod().add("HEAD");
            rule.getAllowedMethod().add("GET");
            rule.getAllowedMethod().add("PUT");
            s3BucketCors.getRules().add(rule);
            
            String requestXml = ConvertUtil.transS3BucketCors(s3BucketCors);
            String requestXmlMd5 = ServiceUtils.computeMD5(requestXml);
            
            Map<String, Object> headers = new HashMap<String, Object>();
            
            headers.put("Content-Type", "application/xml");
            headers.put("Content-MD5", requestXmlMd5);
            
            V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.PUT, 300);
            v4Request.setBucketName(bucketName);
            v4Request.setHeaders(headers);
            v4Request.setSpecialParam(SpecialParamEnum.CORS);
            V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
            
            sb.append("Setting bucket CORS using v4 temporary signature url:");
            sb.append("\t" + v4Response.getSignedUrl() + "\n");
            
            Request.Builder builder = new Request.Builder();
            builder.url(v4Response.getSignedUrl());
            for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
            {
                builder.header(entry.getKey(), entry.getValue().toString());
            }
            builder.put(RequestBody.create(MediaType.parse("application/xml"), requestXml.getBytes("UTF-8")));
            getResponse(builder.build());
            sb.append("\n\n");
            
            v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.GET, 300);
            v4Request.setBucketName(bucketName);
            v4Request.setSpecialParam(SpecialParamEnum.CORS);
            v4Response = obsClient.createV4TemporarySignature(v4Request);
            
            sb.append("Getting bucket CORS using v4 temporary signature url:");
            sb.append("\t" + v4Response.getSignedUrl() + "\n");
            builder = new Request.Builder();
            builder.url(v4Response.getSignedUrl());
            for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
            {
                builder.header(entry.getKey(), entry.getValue().toString());
            }
            getResponse(builder.get().build());
            sb.append("\n\n");
        }
        
        private void doCreateBucket()
            throws ObsException, IOException
        {
            
            V4TemporarySignatureRequest v4Request = new V4TemporarySignatureRequest(HttpMethodEnum.PUT, 300);
            v4Request.setBucketName(bucketName);
            V4TemporarySignatureResponse v4Response = obsClient.createV4TemporarySignature(v4Request);
            sb.append("Creating bucket using v4 temporary signature url:");
            sb.append("\t" + v4Response.getSignedUrl() + "\n");
            
            Request.Builder builder = new Request.Builder();
            builder.url(v4Response.getSignedUrl());
            for (Map.Entry<String, Object> entry : v4Response.getActualSignedRequestHeaders().entrySet())
            {
                builder.header(entry.getKey(), entry.getValue().toString());
            }
            builder.put(RequestBody.create(null, "".getBytes("UTF-8")));
            getResponse(builder.build());
            sb.append("\n\n");
        }
        
        private void getResponse(Request request)
            throws IOException
        {
            Call c = httpClient.newCall(request);
            Response res = c.execute();
            sb.append("\tStatus:" + res.code());
            if (res.body() != null)
            {
                String content = res.body().string();
                if (content == null || content.trim().equals(""))
                {
                    sb.append("\n");
                }
                else
                {
                    sb.append("\tContent:" + content + "\n\n");
                }
            }
            else
            {
                sb.append("\n");
            }
            res.close();
        }
        
    }
    
}
